@@ -1,24 +1,36 @@
 use crate::api::BitcoinDB;
 use crate::parser::blk_file::BlkFile;
 use crate::parser::block_index::BlockIndex;
-use siphasher::sip128::{Hasher128, SipHasher13};
-use std::hash::Hash;
+use ahash::AHasher;
 use bitcoin::Txid;
+use std::collections::VecDeque;
+use std::hash::{Hash, Hasher};
+use std::sync::mpsc::SyncSender;
+use std::sync::{Arc, Mutex};
 
+///
+/// Key compression
+///
 pub(crate) trait Compress {
-    fn compress(&self) -> [u8; 16];
+    fn compress(&self) -> u128;
 }
 
 impl Compress for Txid {
     #[inline]
-    fn compress(&self) -> [u8; 16] {
-        let mut hasher = SipHasher13::new();
-        self.hash(&mut hasher);
-        hasher.finish128().as_bytes()
+    fn compress(&self) -> u128 {
+        let mut hasher_0 = AHasher::new_with_keys(54321, 12345);
+        let mut hasher_1 = AHasher::new_with_keys(12345, 54321);
+        self.hash(&mut hasher_0);
+        self.hash(&mut hasher_1);
+        let hash_0 = (hasher_0.finish() as u128) << 64;
+        let hash_1 = hasher_1.finish() as u128;
+        hash_0 ^ hash_1
     }
 }
 
+///
 /// a light weighted data structure for storing unspent output
+///
 pub(crate) struct VecMap<T> {
     size: u16,
     inner: Box<[Option<T>]>,
@@ -47,9 +59,9 @@ impl<T> VecMap<T> {
     }
 }
 
-/// python iterator implementation does not allow lifetime.
-/// Thus, we must own the necessary resource for the iterator
-/// to work for python.
+///
+/// Each thread owns the necessary resource for better performance.
+///
 #[derive(Clone)]
 pub(crate) struct DBCopy {
     pub block_index: BlockIndex,
@@ -65,6 +77,25 @@ impl DBCopy {
     }
 }
 
+///
+/// Utility function for work stealing.
+/// Exclusive access to task list.
+///
+pub(crate) fn get_task<T>(
+    tasks: &Arc<Mutex<VecDeque<T>>>,
+    register: &SyncSender<usize>,
+    thread_number: usize,
+) -> Option<T> {
+    // lock task list
+    let mut task = tasks.lock().unwrap();
+    let next_height = task.pop_front();
+    // register task stealing
+    if next_height.is_some() {
+        register.send(thread_number).unwrap();
+    }
+    next_height
+}
+
 #[cfg(test)]
 mod test_vec_map {
     use crate::api::STxOut;
